//=======================================================================
/**
 * @file AudioSample.h
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef AUDIOSAMPLE_H
#define AUDIOSAMPLE_H

#include <QObject>

/**
 * @ingroup DataAnalysisReporting
 * @struct AudioSample
 * @brief Represents an audio sample retrieved from a database.
 *
 * It is used to create a list of audio samples based on the database response.
 */
struct AudioSample {
    int sampleId; ///< The unique identifier of the sample in the database.
    QString fileName; ///< The name of the file associated with the sample.
    QString path; ///< The file system path to the audio file.

    /**
     * Constructs a new AudioSample object.
     *
     * This constructor is used when creating AudioSample objects based on
     * a response from the database. Each parameter corresponds to a field
     * in the database.
     *
     * @param sampleId The unique identifier of the sample in the database.
     * @param fileName The name of the file associated with the sample.
     * @param path The file system path to the audio file.
     */
    AudioSample(int sampleId,
                const QString &fileName,
                const QString &path)
        : sampleId(sampleId), fileName(fileName), path(path) {}
};

#endif // AUDIOSAMPLE_H
