//=======================================================================
/**
 * @file CSVExporter.h
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef CSVEXPORTER_H
#define CSVEXPORTER_H

#include <QList>
#include <QString>

#include "TestSessionData.h"

/**
 * @ingroup DataAnalysisReporting
 *
 * @class CSVExporter
 *
 * @brief The CSVExporter class provides functionality to export data to a CSV file.
 *
 * This class allows exporting a QList<TestSessionData> to a CSV file specified by the filePath parameter.
 */
class CSVExporter {
public:
    /**
     * @brief Export data to a CSV file.
     *
     * This function exports the provided data to a CSV file specified by the filePath parameter.
     *
     * @param data The list of TestSessionData to be exported.
     * @param filePath The path of the CSV file to export the data to.
     */
    void exportToCSV(const QList<TestSessionData>& data, const QString& filePath);
};

#endif // CSVEXPORTER_H
