//=======================================================================
/**
 * @file FrequencyResponseHelper.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef FREQUENCYRESPONSEHELPER_H
#define FREQUENCYRESPONSEHELPER_H

#include <vector>
#include <complex>

/**
 * @ingroup Utils
 * @class FrequencyResponseHelper
 * @brief Provides methods to help calculate the frequency response of a filter.
 */
class FrequencyResponseHelper {
public:
    /**
     * @brief Creates a logarithmically spaced vector.
     * @param start The start value of the vector.
     * @param end The end value of the vector.
     * @param num_points The number of points in the vector.
     * @return A logarithmically spaced vector.
     */
    static std::vector<double> logspace(double start, double end, int num_points);

    /**
     * @brief Calculates the frequency response of a filter given its coefficients.
     * @param B The B coefficients of the filter.
     * @param A The A coefficients of the filter.
     * @param sample_rate The sample rate used in the filter.
     * @param frequencies The frequencies at which to calculate the response.
     * @return A vector containing the gain in decibels at each frequency.
     */
    static std::vector<double> calculate_frequency_response(const std::vector<double> &B, const std::vector<double> &A, double sample_rate, const std::vector<double> &frequencies);
};

#endif // FREQUENCYRESPONSEHELPER_H
