//=======================================================================
/**
 * @file SessionTrial.h
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================


#ifndef SESSIONTRIAL_H
#define SESSIONTRIAL_H

#include <QObject>

/**
 * @ingroup DataAnalysisReporting
 *
 * @struct SessionTrial
 *
 * @brief Represents a single trial within a testing session.
 *
 * This struct is used to record the details of a single trial within a testing session.
 * Each trial has a unique trial ID within the context of the session.
 * It also records the correct response for the trial, the user's actual response,
 * and the time taken by the user to respond.
 */
struct SessionTrial {
    int trialId;              ///< The unique identifier for this trial within the session.
    int sessionId;            ///< The identifier of the session this trial belongs to.
    int responseTime;         ///< The time taken by the user to respond, in milliseconds.
    QString correctResponse;  ///< The correct response for this trial.
    QString userResponse;     ///< The user's actual response for this trial.
};

#endif // SESSIONTRIAL_H

