//=======================================================================
/**
 * @file TestSessionData.cpp
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef TESTSESSIONDATA_H
#define TESTSESSIONDATA_H

#include <QString>

/**
 * @ingroup DataAnalysisReporting
 *
 * @struct TestSessionData
 *
 * @brief Contains data for a testing session.
 *
 * This structure is used to hold information about a testing session,
 * including the session ID, start and end dates, sample file name,
 * session duration, trial ID, correct response, user response, and response time.
 */
struct TestSessionData {
    int sessionId; /**< Unique identifier for the session. */
    QString sessionStartDate; /**< Start date of the session. */
    QString sessionEndDate; /**< End date of the session. */
    QString sampleFileName; /**< Name of the sample file used in the session. */
    int sessionDuration; /**< Duration of the session in seconds. */
    int trialId; /**< Unique identifier for the trial within the session. */
    QString correctResponse; /**< The correct response for the trial. */
    QString userResponse; /**< The user's response for the trial. */
    int responseTime; /**< Time taken by the user to respond, in milliseconds. */
};

#endif // TESTSESSIONDATA_H

