//=======================================================================
/**
 * @file Window.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef WINDOW_H
#define WINDOW_H

#include <QWidget>
#include <QStackedWidget>
#include <QPushButton>

#include "TrainingWindow.h"
#include "TestingWindow.h"
#include "StatisticsWindow.h"
#include "DatabaseManager.h"

/**
 * @ingroup UserInterface
 * @class Window
 * @brief Represents the main application window.
 *
 * This class is responsible for managing the main application window and
 * provides methods for showing different sections of the application such as
 * the training window, testing window, and statistics window.
 */
class Window : public QWidget {
    Q_OBJECT

public:
    explicit Window(QWidget *parent = nullptr);  ///< Constructs a Window with an optional parent widget.
    ~Window();  ///< Destructor.

private slots:
    void showTrainingWindow();  ///< Shows the training window.
    void showTestingWindow();   ///< Shows the testing window.
    void showStatisticsWindow(); ///< Shows the statistics window.

private:
    void setUpDatabaseConnection();  ///< Sets up the database connection.
    void updateButtonStyles(QPushButton* selectedButton);  ///< Updates the styles of the buttons.

    QStackedWidget contentWidgetContainer;  ///< The container for the content widgets.

    QPushButton *trainingWindowButton;  ///< The button for the training window.
    QPushButton *testingWindowButton;   ///< The button for the testing window.
    QPushButton *statisticsWindowButton; ///< The button for the statistics window.

    const QString selectedStyleSheet = "font-size: 16pt; font-weight: semi-bold; background-color: #3A3A3C; color: #ffffff; border: 0.5px solid #67676A; padding: 20px; ";
    const QString notSelectedStyleSheet = "font-size: 16pt; font-weight: regular; background-color: #383961; color: #AEAEB2; border: 0.5px solid #67676A; padding: 20px; ";
    const int buttonWidth = 200;

    TrainingWindow *trainingWindow;  ///< The training window.
    TestingWindow *testingWindow;    ///< The testing window.
    StatisticsWindow *statisticsWindow; ///< The statistics window.

    DatabaseManager *dbManager;  ///< The database manager.
};

#endif // WINDOW_H
