//=======================================================================
/**
 * @file BinomialGraph.cpp
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023 Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "BinomialGraph.h"

#include <cmath>

#include <QBarCategoryAxis>
#include <QBarSeries>
#include <QBarSet>
#include <QChart>
#include <QChartView>
#include <QValueAxis>

#include "BinomialHelper.h"


BinomialGraph::BinomialGraph(QWidget *parent)
    : QChartView(parent)
{
    setRenderHint(QPainter::Antialiasing);
}


void BinomialGraph::updateGraph(const QVector<SessionTrial> &trials) {
    QChart *binomialChart = createBinomialChart(trials);
    setChart(binomialChart);
}


QChart *BinomialGraph::createBinomialChart(const QVector<SessionTrial> &trials) {
    int n = trials.size();
    int k = 0;

    for (const SessionTrial &trial : trials) {
        if (trial.correctResponse == trial.userResponse) {
            k++;
        }
    }

    double p = 0.5; // Assuming a listener is guessing

    // Create a bar series for the binomial distribution
    QBarSeries *series = new QBarSeries();

    for (int i = 0; i < n; ++i) {
        double probability = BinomialHelper().binomialProbability(n, i, p);
        QBarSet *barSet = new QBarSet(QString::number(i));
        barSet->append(probability);
        series->append(barSet);
    }

    // Create a chart and add the bar series
    QChart *chart = new QChart();
    chart->addSeries(series);

    // Customize chart title, legend, and axes
    chart->setTitle("Binomial Distribution of ABX Test Results");
    chart->setAnimationOptions(QChart::SeriesAnimations);
    QBarCategoryAxis *axisX = new QBarCategoryAxis();
    axisX->setTitleText("Number of Correct Identifications (k)");
    chart->addAxis(axisX, Qt::AlignBottom);
    series->attachAxis(axisX);

    QValueAxis *axisY = new QValueAxis();
    axisY->setTitleText("Probability");
    chart->addAxis(axisY, Qt::AlignLeft);
    series->attachAxis(axisY);

    // Customize chart appearance
    chart->legend()->setVisible(true);
    chart->legend()->setAlignment(Qt::AlignBottom);

    return chart;
}
