//=======================================================================
/**
 * @file FileListWidget.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "FileListWidget.h"

#include <QScrollArea>
#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>

FileListWidget::FileListWidget(QWidget *parent) : QWidget(parent)
{

}

FileListWidget::~FileListWidget()
{
    // No op yet.
}

QWidget *FileListWidget::createWidget()
{
    QWidget* statisticsWidget = new QWidget();
    QVBoxLayout* contentLayout = new QVBoxLayout(statisticsWidget);

    scrollArea = new QScrollArea();
    scrollArea->setWidgetResizable(true);
    scrollArea->setFixedWidth(250);
    scrollArea->setFixedHeight(470);

    QLabel* titleLabel = new QLabel("Imported audio files");
    titleLabel->setStyleSheet("color: white; font-size: 12pt; font-weight: bold");

    QLabel* subtitleLabel = new QLabel("Select audio file from below to start training");
    subtitleLabel->setStyleSheet("color: white;");
    subtitleLabel->setWordWrap(true);

    scrollAreaContentWidget = new QWidget;
    scrollLayout = new QVBoxLayout(scrollAreaContentWidget);
    scrollLayout->setAlignment(Qt::AlignTop);
    scrollArea->setWidget(scrollAreaContentWidget);

    QPushButton* importButton = new QPushButton("Import file");
    importButton->setStyleSheet("color: #FFFFFF; padding: 4px");
    importButton->setFixedWidth(250);
    connect(importButton, &QPushButton::clicked, [this]() {
        emit didPushImportFileButton();
    });

    QPushButton* clearButton = new QPushButton("Clear files");
    clearButton->setStyleSheet("color: #FFFFFF; padding: 4px");
    clearButton->setFixedWidth(250);
    connect(clearButton, &QPushButton::clicked, [this]() {
        emit didPushClearFileButton();
    });

    contentLayout->addWidget(titleLabel);
    contentLayout->addWidget(subtitleLabel);
    contentLayout->addSpacing(10);
    contentLayout->addWidget(scrollArea);
    contentLayout->addSpacing(20);
    contentLayout->addWidget(importButton);
    contentLayout->addSpacing(10);
    contentLayout->addWidget(clearButton);

    return statisticsWidget;
}


void FileListWidget::populateList(QList<AudioSample> files)
{
    clearList();

    for (int i = 0; i < files.size(); ++i) {
        QLabel* nameLabel = new QLabel(files[i].fileName);

        nameLabel->setProperty("filePath", files[i].path);
        nameLabel->installEventFilter(this);
        if (i == 0) {
            nameLabel->setStyleSheet("color: white; background-color: green; padding: 4px");
            m_currentSelection = nameLabel;
        } else {
            nameLabel->setStyleSheet("color: white; padding: 4px");
        }

        m_labels.append(nameLabel);

        QFrame *line = new QFrame;
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);
        QVBoxLayout* textLayout = new QVBoxLayout();
        textLayout->addWidget(nameLabel);
        textLayout->addWidget(line);
        scrollLayout->addLayout(textLayout);
    }
}


void FileListWidget::clearList() {
    if (scrollAreaContentWidget != nullptr) {
        delete scrollAreaContentWidget;
    }

    scrollAreaContentWidget = new QWidget();
    scrollLayout = new QVBoxLayout(scrollAreaContentWidget);
    scrollLayout->setAlignment(Qt::AlignTop);
    scrollArea->setWidget(scrollAreaContentWidget);

    m_labels.clear();
}


bool FileListWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == QEvent::MouseButtonRelease) {
        QLabel *clickedLabel = qobject_cast<QLabel*>(obj);
        if (clickedLabel && m_currentSelection != clickedLabel) {
            m_currentSelection->setStyleSheet("color: white; padding: 4px");
            clickedLabel->setStyleSheet("color: white; background-color: green; padding: 4px");
            m_currentSelection = clickedLabel;
            emit didSelectAudioFile(m_currentSelection->property("filePath").toString());
        }
    }
    return QWidget::eventFilter(obj, event);
}

QString FileListWidget::getSelectedFilePath()
{
    if (m_currentSelection) {
        return m_currentSelection->property("filePath").toString();
    }
    return QString();
}
