//=======================================================================
/**
 * @file SessionListWidget.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "SessionListWidget.h"

#include <QScrollArea>
#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>


SessionListWidget::SessionListWidget(QWidget *parent) : QWidget(parent)
{
    // No op.
}

SessionListWidget::~SessionListWidget()
{
    // No op yet.
}

QWidget *SessionListWidget::createWidget()
{
    QWidget* statisticsWidget = new QWidget();
    QVBoxLayout* contentLayout = new QVBoxLayout(statisticsWidget);

    scrollArea = new QScrollArea();
    scrollArea->setWidgetResizable(true);
    scrollArea->setFixedWidth(250);
    scrollArea->setFixedHeight(470);

    QLabel* titleLabel = new QLabel("Testing Sessions");
    titleLabel->setStyleSheet("color: white; font-size: 12pt; font-weight: bold");

    QLabel* subtitleLabel = new QLabel("Select session from below to see the statistics.");
    subtitleLabel->setStyleSheet("color: white;");
    subtitleLabel->setWordWrap(true);

    scrollAreaContentWidget = new QWidget;
    scrollLayout = new QVBoxLayout(scrollAreaContentWidget);
    scrollLayout->setAlignment(Qt::AlignTop);
    scrollArea->setWidget(scrollAreaContentWidget);

    contentLayout->addWidget(titleLabel);
    contentLayout->addWidget(subtitleLabel);
    contentLayout->addSpacing(10);
    contentLayout->addWidget(scrollArea);

    return statisticsWidget;
}

int SessionListWidget::getSelectedSessionId() const
{
    if (m_currentSelection) {
        return m_currentSelection->property("filePath").toInt();
    }
    return -1;
}

void SessionListWidget::populateList(QVector<TestingSession> sessions)
{

    clearList();

    for (int i = 0; i < sessions.size(); ++i) {
        QLabel* idLabel = new QLabel;
        QString formattedId = QString("Session: #%1").arg(sessions[i].sessionId);
        idLabel->setText(formattedId);

        QLabel* durationLabel = new QLabel(QString("Duration: %1").arg(formatDuration(sessions[i].duration)));
        durationLabel->setStyleSheet("padding: 4px");

        idLabel->setProperty("sessionId", sessions[i].sessionId);
        idLabel->installEventFilter(this);
        if (i == 0) {
            idLabel->setStyleSheet("color: white; background-color: green; padding: 4px");
            m_currentSelection = idLabel;
        } else {
            idLabel->setStyleSheet("color: white; padding: 4px");
        }
        m_labels.append(idLabel);

        QFrame *line = new QFrame;
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);
        QVBoxLayout* textLayout = new QVBoxLayout();
        textLayout->addWidget(idLabel);
        textLayout->setSpacing(0);
        textLayout->addWidget(durationLabel);
        textLayout->setSpacing(1);
        textLayout->addWidget(line);

        scrollLayout->addLayout(textLayout);
    }
}

void SessionListWidget::clearList()
{
    if (scrollAreaContentWidget != nullptr) {
        delete scrollAreaContentWidget;
    }

    scrollAreaContentWidget = new QWidget();
    scrollLayout = new QVBoxLayout(scrollAreaContentWidget);
    scrollLayout->setAlignment(Qt::AlignTop);
    scrollArea->setWidget(scrollAreaContentWidget);

    m_labels.clear();
}

bool SessionListWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == QEvent::MouseButtonRelease) {
        QLabel *clickedLabel = qobject_cast<QLabel*>(obj);
        if (clickedLabel && m_currentSelection != clickedLabel) {
            m_currentSelection->setStyleSheet("color: white; padding: 4px");
            clickedLabel->setStyleSheet("color: white; background-color: green; padding: 4px");
            m_currentSelection = clickedLabel;
            emit didSelectTestingSession(m_currentSelection->property("sessionId").toInt());
        }
    }
    return QWidget::eventFilter(obj, event);
}

QString SessionListWidget::formatDuration(int durationInSeconds) const
{
    int hours = durationInSeconds / 3600;
    int minutes = (durationInSeconds % 3600) / 60;
    int seconds = durationInSeconds % 60;

    QString formattedDuration;
    if (hours > 0) {
        formattedDuration.append(QString("%1 hr ").arg(hours));
    }
    if (minutes > 0) {
        formattedDuration.append(QString("%1 m ").arg(minutes));
    }
    formattedDuration.append(QString("%1 s").arg(seconds));

    return formattedDuration;
}
