//=======================================================================
/**
 * @file StatisticsSummaryWidget.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================


#include "StatisticsSummaryWidget.h"

#include <QGridLayout>

QString formatSeconds(int durationInSeconds) {
    int hours = durationInSeconds / 3600;
    int minutes = (durationInSeconds % 3600) / 60;
    int seconds = durationInSeconds % 60;

    QString formattedDuration;
    if (hours > 0) {
        formattedDuration.append(QString("%1 hr ").arg(hours));
    }
    if (minutes > 0) {
        formattedDuration.append(QString("%1 m ").arg(minutes));
    }
    formattedDuration.append(QString("%1 s").arg(seconds));

    return formattedDuration;
}


StatisticsSummaryWidget::StatisticsSummaryWidget(QWidget *parent)
    : QWidget(parent)
{
    // Create layout for the widget
    QHBoxLayout *layout = new QHBoxLayout;

    // Create the boxes with statistics
    layout->addLayout(createStatisticBox("Average Trials/Session", &averageTrialsLabel));
    averageTrialsLabel->setAlignment(Qt::AlignCenter);
    layout->addSpacing(20);
    layout->addLayout(createStatisticBox("Average Response Time/Trial", &averageResponseTimeLabel));
    averageResponseTimeLabel->setAlignment(Qt::AlignCenter);
    layout->addSpacing(20);
    layout->addLayout(createStatisticBox("Average Duration/Session", &averageDurationLabel));
    averageDurationLabel->setAlignment(Qt::AlignCenter);
    layout->addSpacing(20);
    layout->addLayout(createStatisticBox("Average Success/Session", &averageSuccessLabel));
    averageSuccessLabel->setAlignment(Qt::AlignCenter);

    // Set the layout
    setLayout(layout);
}

QVBoxLayout *StatisticsSummaryWidget::createStatisticBox(const QString &title, QLabel **valueLabel)
{
    // Create group box with a vertical layout
    QVBoxLayout *layout = new QVBoxLayout;

    // title
    QLabel* titleLabel = new QLabel(title);

    // Add subtitle label to layout
    *valueLabel = new QLabel;
    layout->addWidget(titleLabel);
    layout->addSpacing(20);
    layout->addWidget(*valueLabel);

    return layout;
}

void StatisticsSummaryWidget::updateAverageTrials(double value) {
    averageTrialsLabel->setText(QString::number(value));
}

void StatisticsSummaryWidget::updateAverageResponseTime(double value) {
    QString formattedTime = formatSeconds(static_cast<int>(value));
    averageResponseTimeLabel->setText(formattedTime);
}

void StatisticsSummaryWidget::updateAverageDuration(double value) {
    QString formattedTime = formatSeconds(static_cast<int>(value));
    averageDurationLabel->setText(formattedTime);
}

void StatisticsSummaryWidget::updateAverageSuccess(double value) {
    int successRate = static_cast<int>(value);  // convert fraction to percentage
    averageSuccessLabel->setText(QString::number(successRate) + "%");
}
