//=======================================================================
/**
 * @file TestingWindow.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "TestingWindow.h"

#include <random>
#include <ctime>

#include <QLabel>
#include <QPushButton>
#include <QVBoxLayout>
#include <QComboBox>
#include <QSpinBox>
#include <QThread>
#include <QMessageBox>

#include "CustomComboBox.h"

// TODO: Cleanup and refactor this class.
TestingWindow::TestingWindow(QWidget *parent) : QWidget(parent)
{
    // No op.
}

TestingWindow::~TestingWindow()
{
    // No op yet.
}

QWidget *TestingWindow::createWidget()
{
    QWidget* testingWidget = new QWidget();

    // Control Panel
    QLabel* controlPanelTitle = new QLabel("Control Panel");
    controlPanelTitle->setAlignment(Qt::AlignLeading);
    controlPanelTitle->setStyleSheet("color: white; font-size: 12pt; font-weight: bold");
    QFont f = controlPanelTitle->font();
    f.setUnderline(true);
    controlPanelTitle->setFont(f);

    // Create two buttons vertically centered
    QLabel* statusLabel = new QLabel("Status: Stopped");
    startStopButton = new QPushButton("Start Session");

    QString buttonsStyleSheet = "color: #FFFFFF; padding: 4px" ;

    QVBoxLayout* buttonLayout = new QVBoxLayout;
    buttonLayout->addStretch();
    buttonLayout->addWidget(statusLabel);
    buttonLayout->addSpacing(8);
    buttonLayout->addWidget(startStopButton);
    buttonLayout->addStretch();

    statusLabel->setStyleSheet("color: white; font-size: 12pt");
    startStopButton->setStyleSheet(buttonsStyleSheet);

    QLabel* trialsNumberLabel = new QLabel("Select number of trials");
    trialsNumberLabel->setStyleSheet("color: white;");
    trialsNumberLabel->setAlignment(Qt::AlignLeading);
    trialsNumberLabel->setFixedHeight(20);

    // Numbe of trials selection spinbox
    numberOfTrialsSpinbox = new QSpinBox;
    numberOfTrialsSpinbox->setMinimum(5); // Set the minimum value
    numberOfTrialsSpinbox->setMaximum(100); // Set the maximum value
    numberOfTrialsSpinbox->setValue(5); // Set the initial value
    numberOfTrialsSpinbox->setFixedSize(250, 30);
    numberOfTrialsSpinbox->setStyleSheet(buttonsStyleSheet);

    connect(numberOfTrialsSpinbox, &QSpinBox::valueChanged, this, [this](int currentValue) {
        maxTrialsCount = currentValue;
        QString formatted = QString("Attempt %1/%2").arg(currentTrialCursor).arg(currentValue);
        attemptCountLabel->setText(formatted);
    });

    QVBoxLayout* trialsSelectionLayout = new QVBoxLayout;
    trialsSelectionLayout->addWidget(trialsNumberLabel);
    trialsSelectionLayout->addWidget(numberOfTrialsSpinbox);

    // Track selection
    QLabel* trackSelectionLabel = new QLabel("Select the audio file");
    trackSelectionLabel->setStyleSheet("color: white;");
    trackSelectionLabel->setAlignment(Qt::AlignLeading);
    trackSelectionLabel->setFixedHeight(20);

    trackSelectionDropDown = new CustomComboBox;

    // Add items to the dropdown
    trackSelectionDropDown->setFixedSize(250, 30);
    trackSelectionDropDown->setStyleSheet(buttonsStyleSheet);

    connect(trackSelectionDropDown, &QComboBox::activated, this, [this] (int index) {
        if (index < audioFiles.size()) {
            didSelectAudioFile(audioFiles[index]);
        }
    });

    QVBoxLayout* trackSelectionLayout = new QVBoxLayout;
    trackSelectionLayout->addWidget(trackSelectionLabel);
    trackSelectionLayout->addWidget(trackSelectionDropDown);

    QVBoxLayout* controlPanelLayout = new QVBoxLayout;
    controlPanelLayout->addSpacing(20);
    controlPanelLayout->addWidget(controlPanelTitle);
    controlPanelLayout->addSpacing(20);
    controlPanelLayout->addLayout(buttonLayout);
    controlPanelLayout->addSpacing(40);
    controlPanelLayout->addLayout(trialsSelectionLayout);
    controlPanelLayout->addSpacing(20);
    controlPanelLayout->addLayout(trackSelectionLayout);
    controlPanelLayout->addStretch();

    // Testing panel

    // Current attempt status
    QString formatted = QString("Attempt %1/%2").arg(currentTrialCursor + 1).arg(maxTrialsCount);
    attemptCountLabel = new QLabel(formatted);
    attemptCountLabel->setStyleSheet("color: white; font-weight: bold");
    attemptCountLabel->setAlignment(Qt::AlignHCenter);
    attemptCountLabel->setFixedHeight(20);

    // Currently playing
    currentlyPlayinglabel = new QLabel("Currently Playing: None");
    currentlyPlayinglabel->setStyleSheet("color: white;");
    currentlyPlayinglabel->setAlignment(Qt::AlignHCenter);
    currentlyPlayinglabel->setFixedHeight(20);

    // Play buttons
    QPushButton* playAButton = new QPushButton("Play A");
    QPushButton* playBButton = new QPushButton("Play B");
    QPushButton* playXButton = new QPushButton("Play X");

    playAButton->setStyleSheet(buttonsStyleSheet);
    playBButton->setStyleSheet(buttonsStyleSheet);
    playXButton->setStyleSheet(buttonsStyleSheet);

    playAButton->setFixedWidth(100);
    playBButton->setFixedWidth(100);
    playXButton->setFixedWidth(100);

    QHBoxLayout* playButtonLayout = new QHBoxLayout;
    playButtonLayout->addStretch();
    playButtonLayout->addWidget(playAButton);
    playButtonLayout->addSpacing(20);
    playButtonLayout->addWidget(playXButton);
    playButtonLayout->addSpacing(20);
    playButtonLayout->addWidget(playBButton);
    playButtonLayout->addStretch();

    // You answer label
    QLabel* selectTrackLabel = new QLabel("Which one of A or B do you think was X?");
    selectTrackLabel->setStyleSheet("color: white;");
    selectTrackLabel->setAlignment(Qt::AlignHCenter);
    selectTrackLabel->setFixedHeight(20);

    // Response buttons
    QPushButton* responseAButton = new QPushButton("X = A");
    QPushButton* responseBButton = new QPushButton("X = B");

    responseAButton->setStyleSheet(buttonsStyleSheet);
    responseBButton->setStyleSheet(buttonsStyleSheet);

    responseAButton->setFixedWidth(100);
    responseBButton->setFixedWidth(100);

    QHBoxLayout* responseButtonLayout = new QHBoxLayout;
    responseButtonLayout->addStretch();
    responseButtonLayout->addWidget(responseAButton);
    responseButtonLayout->addSpacing(20);
    responseButtonLayout->addWidget(responseBButton);
    responseButtonLayout->addStretch();

    QVBoxLayout* testingPanelLayout = new QVBoxLayout;
    testingPanelLayout->addSpacing(20);
    testingPanelLayout->addWidget(attemptCountLabel);
    testingPanelLayout->addSpacing(30);
    testingPanelLayout->addWidget(currentlyPlayinglabel);
    testingPanelLayout->addSpacing(10);
    testingPanelLayout->addLayout(playButtonLayout);
    testingPanelLayout->addSpacing(60);
    testingPanelLayout->addWidget(selectTrackLabel);
    testingPanelLayout->addSpacing(20);
    testingPanelLayout->addLayout(responseButtonLayout);
    testingPanelLayout->addStretch();

    // Combine both layout
    QHBoxLayout* contentLayout = new QHBoxLayout();
    contentLayout->addSpacing(20);
    contentLayout->addLayout(controlPanelLayout);
    contentLayout->addSpacing(50);
    contentLayout->addLayout(testingPanelLayout);
    contentLayout->addStretch();

    // Main Layout
    QVBoxLayout* mainLayout = new QVBoxLayout(testingWidget);
    mainLayout->addLayout(contentLayout);
    mainLayout->addStretch();
    mainLayout->setContentsMargins(40, 40, 0, 0);

    // TODO: Refactor this
    // THREAD
    QThread::currentThread()->setObjectName("Main Thread");
    qDebug()<<"From main thread: "<<QThread::currentThread()->objectName();

    workerThread = new QThread();
    jackWorker = new JackWorker();
    jackWorker->moveToThread(workerThread);
    // SLOT
    //    connect( worker, &Worker::error, this, &MyClass::errorString);
    connect( workerThread, &QThread::started, jackWorker, &JackWorker::setupJackServer);
    connect( jackWorker, &JackWorker::finished, workerThread, &QThread::quit);
    connect( jackWorker, &JackWorker::finished, jackWorker, &JackWorker::deleteLater);
    connect( workerThread, &QThread::finished, workerThread, &QThread::deleteLater);

    workerThread->start();

    connect(startStopButton, &QPushButton::clicked, [this]() {
        // TODO: Dont' allow session start if audio file is not valid.
        if (jackWorker->isAudioEngineRunning()) {
            if (currentTrialCursor != maxTrialsCount) {
                showSessionEarlyEndConfirmationDialog("You did not attempt all trials. Are you sure?");
            }
        } else {
            showSessionStartConfirmationDialog("The session will have the same filter parameter configugation as your last training session.\nDo you want to start new testing session?");
        }
    });

    connect(jackWorker, &JackWorker::jackEngineStatus, this, [statusLabel, this](QString status) {
        QString statusFormatted = QString("Status: %1").arg(status);
        statusLabel->setText(statusFormatted);
    });

    // Play buttons
    connect(playAButton, &QPushButton::clicked, [this]() {
        // Randomly play modified or reference track when A is tapped.
        if (!jackWorker->isAudioEngineRunning()) {
            showErrorDialog("Please start the session first");
            return;
        } else {
            startPlayingTrackA();
            startResponseTimer();
        }
    });

    connect(playBButton, &QPushButton::clicked, [this]() {
        // Randomly play modified or reference track when B is tapped.
        if (!jackWorker->isAudioEngineRunning()) {
            showErrorDialog("Please start the session first");
            return;
        } else {
            if (buttonBTrackType == TrackType::Modified) {
                qDebug() << "Modified";
            } else {
                qDebug() << "Reference";
            }

            jackWorker->setIsPlayingModified(buttonBTrackType == TrackType::Modified);
            jackWorker->setAudioFile(currentAudioFileData);

            currentlyPlayinglabel->setText("Currently playing: Track B");
            startResponseTimer();

        }
    });

    connect(playXButton, &QPushButton::clicked, [this]() {
        // Play the modified track always here.
        if (!jackWorker->isAudioEngineRunning()) {
            showErrorDialog("Please start the session first");
            return;
        } else {
            jackWorker->setIsPlayingModified(true);
            jackWorker->setAudioFile(currentAudioFileData);

            currentlyPlayinglabel->setText("Currently playing: Track X");
            startResponseTimer();
        }
    });

    // Response Buttons
    connect(responseAButton, &QPushButton::clicked, [this]() {

        if (!jackWorker->isAudioEngineRunning()) {
            showErrorDialog("Please start the session first");
            return;
        } else {

            QString correctResponse = buttonATrackType == TrackType::Modified ? "A" : "B";
            if (dbManager == nullptr)
                return;
            int elapsedTime = responseTimer.isValid() ? responseTimer.elapsed() : 1000;
            bool insertTrial = dbManager->insertTrial(currentSession, correctResponse, "A", elapsedTime);
            if (!insertTrial)
                return;

            if (currentTrialCursor < maxTrialsCount) {
                currentTrialCursor++;

                if (currentTrialCursor == maxTrialsCount) {
                    showSessionStopConfirmationDialog("Testing Session Completed. Navigate to Statistics to check the test results.");
                } else {
                    QString formatted = QString("Attempt %1/%2").arg(currentTrialCursor + 1).arg(maxTrialsCount);
                    attemptCountLabel->setText(formatted);
                    shuffleTracks(buttonATrackType, buttonBTrackType);
                    startPlayingTrackA();
                    restartResponseTimer();
                }
            }
        }
    });

    connect(responseBButton, &QPushButton::clicked, [this]() {
        if (!jackWorker->isAudioEngineRunning()) {
            showErrorDialog("Please start the session first");
            return;
        } else {
            QString correctResponse = buttonATrackType == TrackType::Modified ? "A" : "B";
            if (dbManager == nullptr)
                return;
            int elapsedTime = responseTimer.isValid() ? responseTimer.elapsed() : 1000;
            bool insertTrial = dbManager->insertTrial(currentSession, correctResponse, "B", elapsedTime);
            if (!insertTrial)
                return;

            if (currentTrialCursor < maxTrialsCount) {
                currentTrialCursor++;

                if (currentTrialCursor == maxTrialsCount) {
                    showSessionStopConfirmationDialog("Testing Session Completed. Navigate to Statistics to check the test results.");
                } else {
                    QString formatted = QString("Attempt %1/%2").arg(currentTrialCursor + 1).arg(maxTrialsCount);
                    attemptCountLabel->setText(formatted);
                    shuffleTracks(buttonATrackType, buttonBTrackType);
                    startPlayingTrackA();
                    restartResponseTimer();
                }
            }
        }
    });

    // -----------------------------------------------

    return testingWidget;
}

void TestingWindow::setDatabaseManager(DatabaseManager *manager)
{
    dbManager = manager;
}

void TestingWindow::populateAudioFiles()
{
    if (!dbManager)
        return;

    trackSelectionDropDown->clear();

    audioFiles = dbManager->getAudioFiles();
    for (int i = 0; i < audioFiles.size(); ++i) {
        trackSelectionDropDown->addItem(audioFiles[i].fileName);

        if (audioFiles[i].path == selectedAudioFilePath) {
            trackSelectionDropDown->setCurrentIndex(i);
            trackSelectionDropDown->setCurrentText(audioFiles[i].fileName);
        }
    }

    if (selectedAudioFilePath.isEmpty() && audioFiles.size() > 0) {
        selectedAudioFilePath = audioFiles[0].path;
        selectedAudioFileId = audioFiles[0].sampleId;
    }
}

void TestingWindow::updateSelectedAudioFile(QString filePath)
{
    this->selectedAudioFilePath = filePath;

    for (const AudioSample& sample : audioFiles) {
        if (sample.path == filePath) {
            this->selectedAudioFileId = sample.sampleId;
        }
    }
}

void TestingWindow::updateSelectedFilterType(FilterType filterType)
{
    this->activeFilterType = filterType;
}

void TestingWindow::updateFilterParameters(int gain, int cutOffFrequency)
{
    this->gain = gain;
    this->cutOffFrequency = cutOffFrequency;
}

void TestingWindow::shuffleTracks(TrackType &buttonATrack, TrackType &buttonBTrack)
{
    std::random_device rd; // Obtain a random number from hardware
    std::mt19937 gen(rd()); // Seed the random number generator
    std::uniform_int_distribution<> distr(0, 1); // Define the distribution

    int randomNumber = distr(gen); // Generate a random number (0 or 1)

    buttonATrack = randomNumber == 0 ? TrackType::Reference : TrackType::Modified;
    buttonBTrack = randomNumber == 0 ? TrackType::Modified : TrackType::Reference;
}

void TestingWindow::didSelectAudioFile(AudioSample audioSample)
{
    if (!audioSample.path.isEmpty()) {
        selectedAudioFilePath = audioSample.path;
        selectedAudioFileId = audioSample.sampleId;
    }
}

void TestingWindow::startPlayingTrackA()
{
    if (buttonATrackType == TrackType::Modified) {
        qDebug() << "Modified";
    } else {
        qDebug() << "Reference";
    }

    jackWorker->setIsPlayingModified(buttonATrackType == TrackType::Modified);
    jackWorker->setAudioFile(currentAudioFileData);

    currentlyPlayinglabel->setText("Currently playing: Track A");
}

void TestingWindow::showErrorDialog(const QString &message)
{
    QMessageBox::critical(nullptr, "Error", message);
}

void TestingWindow::showSessionStartConfirmationDialog(const QString &message)
{
    QMessageBox msgBox;
    msgBox.setWindowTitle("Confirmation");
    msgBox.setText(message);

    QPushButton *actionButton = msgBox.addButton("Start session", QMessageBox::AcceptRole);
    msgBox.addButton("Close", QMessageBox::RejectRole);

    msgBox.exec();

    if (msgBox.clickedButton() == actionButton) {
        startTestingSessionAction();
    }
}

void TestingWindow::showSessionStopConfirmationDialog(const QString &message)
{
    QMessageBox msgBox;
    msgBox.setWindowTitle("Confirmation");
    msgBox.setText(message);

    QPushButton *actionButton = msgBox.addButton("Close", QMessageBox::AcceptRole);

    msgBox.exec();

    if (msgBox.clickedButton() == actionButton) {
        stopTestingSessionAction();
    }
}

void TestingWindow::showSessionEarlyEndConfirmationDialog(const QString &message)
{
    QMessageBox msgBox;
    msgBox.setWindowTitle("Confirmation");
    msgBox.setText(message);

    QPushButton *actionButton = msgBox.addButton("Stop session", QMessageBox::AcceptRole);
    msgBox.addButton("Close", QMessageBox::RejectRole);

    msgBox.exec();

    if (msgBox.clickedButton() == actionButton) {
        stopTestingSessionAction();
    }
}

void TestingWindow::startTestingSessionAction()
{
    if (!jackWorker->isAudioEngineRunning()) {

        bool loadAudioFile = loadSelectedAudioFile(selectedAudioFilePath);
        if (!loadAudioFile) {
            showErrorDialog("File not loaded. Please select a different file.");
        } else {
            if (dbManager == nullptr)
                return;
            int sessionId = dbManager->createTestSession(selectedAudioFileId, maxTrialsCount);
            if (sessionId == -1)
                return;
            this->currentSession = sessionId;

            currentTrialCursor = 0;
            QString formatted = QString("Attempt %1/%2").arg(currentTrialCursor + 1).arg(maxTrialsCount);
            attemptCountLabel->setText(formatted);

            jackWorker->startJackServer();
            jackWorker->setFilterConfig(cutOffFrequency, gain);
            startStopButton->setText("Stop Session");

            shuffleTracks(buttonATrackType, buttonBTrackType);

            numberOfTrialsSpinbox->setEnabled(false);
            numberOfTrialsSpinbox->setToolTip("Please stop the testing session to change trials number.");
            trackSelectionDropDown->setEnabled(false);
            trackSelectionDropDown->setToolTip("Please stop the testing session to change audio file.");

            emit didChangeTestingSessionState(SessionType::Running);
            responseTimer.invalidate();
        }
    }
}

void TestingWindow::stopTestingSessionAction()
{
    if (jackWorker->isAudioEngineRunning()) {
        if (dbManager == nullptr || currentSession == -1)
            return;
        bool updateSession = dbManager->updateTestSessionEndTime(currentSession);
        if (!updateSession)
            return;

        jackWorker->stopJackServer();
        startStopButton->setText("Start Session");
        numberOfTrialsSpinbox->setEnabled(true);
        numberOfTrialsSpinbox->setToolTip("");
        trackSelectionDropDown->setEnabled(true);
        trackSelectionDropDown->setToolTip("");

        currentTrialCursor = 0;
        QString formatted = QString("Attempt %1/%2").arg(currentTrialCursor + 1).arg(maxTrialsCount);
        attemptCountLabel->setText(formatted);

        currentlyPlayinglabel->setText("Currently playing: None");

        emit didChangeTestingSessionState(SessionType::Stopped);

        responseTimer.invalidate();
    }
}

void TestingWindow::startResponseTimer()
{
    if (!responseTimer.isValid()) {
        responseTimer.start();
    }
}

void TestingWindow::restartResponseTimer()
{
    qint64 elapsedTime = responseTimer.elapsed();
    responseTimer.restart();
}

bool TestingWindow::loadSelectedAudioFile(QString filePath)
{

    AudioFile<float> a;
    bool loadedOK = a.load (filePath.toStdString());

    if (!loadedOK)
        return false; // TODO: Show an error here.
    currentAudioFileData = a;

    return true;
}
