#include "inversebinomialgraph.h"
#include <cmath>

#include <QtCharts>
#include <QBarSeries>
#include <QBarSet>
#include <QChart>
#include <QChartView>
#include <QBarCategoryAxis>
#include <QValueAxis>

#include "BinomialHelper.h"

// TODO: @priyo: Remove this class and extend the binomial graph class to create and render this.
InverseBinomialGraph::InverseBinomialGraph(QWidget *parent)
    : QChartView(parent)
{
    setRenderHint(QPainter::Antialiasing);
}

void InverseBinomialGraph::updateGraph(const QVector<SessionTrial> &trials) {
    QChart *inverseBinomialChart = createInverseBinomialChart(trials);
    setChart(inverseBinomialChart);
}

QChart *InverseBinomialGraph::createInverseBinomialChart(const QVector<SessionTrial> &trials) {
    int n = trials.size();
    double p = 0.5; // Assuming a listener is guessing

    // Create a bar series for the complementary cumulative distribution
    QBarSeries *series = new QBarSeries();

    for (int k = 0; k < n; ++k) {
        double probability = 1.0 - BinomialHelper::binomialCDF(n, k - 1, p);
        QBarSet *barSet = new QBarSet(QString::number(k));
        barSet->append(probability);
        series->append(barSet);
    }

    // Create a chart and add the bar series
    QChart *chart = new QChart();
    chart->addSeries(series);

    // Customize chart title, legend, and axes
    chart->setTitle("Complementary Cumulative Probability of Correct Identifications in ABX Test");
    chart->setAnimationOptions(QChart::SeriesAnimations);

    QBarCategoryAxis *axisX = new QBarCategoryAxis();
    axisX->setTitleText("Number of Correct Identifications (k)");
    chart->addAxis(axisX, Qt::AlignBottom);
    series->attachAxis(axisX);

    QValueAxis *axisY = new QValueAxis();
    axisY->setTitleText("Probability");
    chart->addAxis(axisY, Qt::AlignLeft);
    series->attachAxis(axisY);

    // Customize chart appearance
    chart->legend()->setVisible(true);
    chart->legend()->setAlignment(Qt::AlignBottom);

    return chart;
}

