#include "BenchmarkJackClientAudioEngine.h"

#include "FilterType.h"

void BenchmarkJackClientAudioEngine::initTestCase()
{
    jackClientAudioEngine = new JackClientAudioEngine();
}

void BenchmarkJackClientAudioEngine::cleanupTestCase()
{
    delete jackClientAudioEngine;
}

void BenchmarkJackClientAudioEngine::benchmarkAudioCallback()
{
    auto engine = std::make_shared<JackClientAudioEngine>();

    const jack_nframes_t nframes = 256;
    const unsigned int num_channels = 2;  // Assume stereo audio

    // Create mock input and output buffers.
    JackClientAudioEngine::audioBufVector inBufs(num_channels);
    JackClientAudioEngine::audioBufVector outBufs(num_channels);

    engine->setAudioSource(SourceType::JackInput);

    // Fill in the buffers
    for (unsigned int i = 0; i < num_channels; ++i) {
        inBufs[i] = new float[nframes]();
        outBufs[i] = new float[nframes]();
    }

    engine->setIsPlayingModified(true);  // Set this to true to have the filter process the audio.

    QBENCHMARK {
        engine->audioCallback(nframes, inBufs, outBufs);
    }

    // Free the allocated memory!
    for (unsigned int i = 0; i < num_channels; ++i) {
        delete[] inBufs[i];
        delete[] outBufs[i];
    }
}

void BenchmarkJackClientAudioEngine::benchmarkAudioCallbackWithFile()
{
    // Load the audio file
    AudioFile<float> audioFile;
    // There must be a file at this path.
    bool loadedOK = audioFile.load("/home/ipriyonto/Desktop/Trial Audio/file_example.wav");

    // Check that the file was loaded successfully
    if (!loadedOK) {
        QFAIL("Failed to load audio file");
        return;
    }

    const jack_nframes_t nframes = 256; // Buffer size
    auto engine = std::make_shared<JackClientAudioEngine>();
    engine->setAudioFile(audioFile);
    engine->setAudioSource(SourceType::Local);
    const unsigned int num_channels = 2;  // Assume stereo audio

    // Create mock input and output buffers.
    JackClientAudioEngine::audioBufVector inBufs(num_channels);
    JackClientAudioEngine::audioBufVector outBufs(num_channels);

    engine->setIsPlayingModified(true);  // Set this to true to have the filter process the audio.

    // Fill in the buffers
    for (unsigned int i = 0; i < num_channels; ++i) {
        inBufs[i] = new float[nframes]();
        outBufs[i] = new float[nframes]();
    }

    QBENCHMARK {
        // Run the audio outBufs
        engine->audioCallback(nframes, inBufs, outBufs);
    }

    // Free the allocated memory!
    for (unsigned int i = 0; i < num_channels; ++i) {
        delete[] inBufs[i];
        delete[] outBufs[i];
    }
}

void BenchmarkJackClientAudioEngine::benchmarkSetVolumeLevel()
{
    QBENCHMARK {
        jackClientAudioEngine->setVolumeLevel(99);
    }
}

void BenchmarkJackClientAudioEngine::benchmarkSetFilterConfig()
{
    QBENCHMARK {
        jackClientAudioEngine->setFilterConfig(2000, 5);
    }
}

void BenchmarkJackClientAudioEngine::benchmarkSetActiveFilterType()
{
    QBENCHMARK {
        jackClientAudioEngine->setActiveFilterType(FilterType::HighShelf);
    }
}

#include "BenchmarkJackClientAudioEngine.moc"
