#include "TestBinomialHelper.h"

void TestBinomialHelper::testLogFactorial() {
    QCOMPARE(BinomialHelper::logFactorial(5), std::log(120));
    QCOMPARE(BinomialHelper::logFactorial(10), std::log(3628800));
    QCOMPARE(BinomialHelper::logFactorial(0), 0.0);
}

void TestBinomialHelper::testLogCombination() {
    QVERIFY(qFuzzyCompare(BinomialHelper::logCombination(10, 5), std::log(252)));
    QVERIFY(qFuzzyCompare(BinomialHelper::logCombination(6, 3), std::log(20)));
    QVERIFY(qFuzzyCompare(BinomialHelper::logCombination(10, 10), 0.0));
}

void TestBinomialHelper::testCombination() {
    QCOMPARE(BinomialHelper::combination(10, 5), 252);
    QCOMPARE(BinomialHelper::combination(6, 3), 20);
    QCOMPARE(BinomialHelper::combination(10, 10), 1);
}

void TestBinomialHelper::testBinomialProbability() {
    QVERIFY(qFuzzyCompare(BinomialHelper::binomialProbability(5, 3, 0.5), 0.3125));
    QVERIFY(qFuzzyCompare(BinomialHelper::binomialProbability(5, 2, 0.5), 0.3125));
    QVERIFY(qFuzzyCompare(BinomialHelper::binomialProbability(5, 0, 0.5), std::pow(0.5, 5)));
}

void TestBinomialHelper::testBinomialCDF() {
    QVERIFY(qFuzzyCompare(BinomialHelper::binomialCDF(5, 3, 0.5), 0.8125));
    QVERIFY(qFuzzyCompare(BinomialHelper::binomialCDF(10, 2, 0.1), 0.9298091736));
    QVERIFY(qFuzzyCompare(BinomialHelper::binomialCDF(5, 0, 0.5), BinomialHelper::binomialProbability(5, 0, 0.5)));
}

void TestBinomialHelper::testInverseBinomialCDF() {
    QCOMPARE(BinomialHelper::inverseBinomialCDF(0.6875, 5, 0.5), 3);
    QCOMPARE(BinomialHelper::inverseBinomialCDF(0.95, 10, 0.5), 8);
    QCOMPARE(BinomialHelper::inverseBinomialCDF(0.1, 10, 0.1), 0);
}
